const APPNAME: &'static str = "Source Cubemapper (Client)";

extern crate clap;

use clap::{App, Arg};
use core::time;
use interprocess::local_socket::LocalSocketStream;
use bincode::{serialize, deserialize};
use source_cubemapper::*;
use std::{
    io::{BufRead, BufReader, ErrorKind, Write},
    process::exit,
    thread::sleep,
};

fn main() -> Result<()> {
    /////////////////////////////////////////////////////////////////////////
    // CLI
    let long_version = format!(
        "\
        \nRevision: ({})\
        \nDate:     {}\
        \nVersion:  {}\
        \nBuilt:    {}",
        REVISION, REVDATE, VERSION, BUILDTD,
    );

    let matches = App::new(APPNAME)
        .version(&*REVISION)
        .long_version(&*long_version)
        .author(AUTHORS)
        .long_about(
            format!(
                "{}\n \
            Submits a job to the cubemap server and waits for a response.",
                APPNAME
            )
            .as_str(),
        )
        .arg(
            Arg::with_name("map")
                .long("map")
                .takes_value(true)
                .required(true),
        )
        .arg(
            Arg::with_name("hdr")
                .help("Builds HDR Cubemaps")
                .takes_value(true)
                .required_unless("ldr"),
        )
        .arg(
            Arg::with_name("ldr")
                .help("Builds LDR Cubemaps")
                .takes_value(true)
                .required_unless("hdr"),
        )
        .get_matches();
    // CLI
    /////////////////////////////////////////////////////////////////////////
    let map = matches
        .value_of_os("map")
        .unwrap()
        .to_str()
        .unwrap()
        .to_string();
    let hdr = matches.value_of_os("hdr").is_some();
    let ldr = matches.value_of_os("ldr").is_some();
    let job = Job {
        map,
        dnr: DNR { HDR: hdr, LDR: ldr },
    };
    // Make connection
    println!("Attempting connection to Server");
    let mut iters = 0;
    let mut conn;
    loop {
        iters = iters + 1;
        let attempt = LocalSocketStream::connect(SOCK_LOCATION);
        match attempt {
            Ok(v) => {
                println!("Connected!");
                conn = v;
                break;
            }
            Err(e) => {
                //println!("Kind: {:?}", e.kind());
                if e.kind() == ErrorKind::NotFound {
                    eprintln!("Could not connect to server!");
                    exit(1);
                }
                if iters == 1 || iters % 30 == 0 {
                    println!("\t Retrying connection... {}", iters)
                }
                sleep(time::Duration::from_millis(500));
                continue;
            }
        }
    }
    // Submit Job
    let job_msg = Message::Job(job);
    let mut job_ser = serialize(&job_msg).unwrap();
    job_ser.push(0xA);
    conn.write_all(job_ser.as_ref())?;
    // Purely getting responses now
    let mut reader = BufReader::new(conn);
    let mut buffer = Vec::new();
    reader.read_until(0xA, &mut buffer)?;
    buffer.pop();
    // This should really be a message state machine
    // But uh, we can kindly call this "manually unrolled"
    // "If it works it's not a hack" - Colin
    // - Glitchvid
    let resp = deserialize::<Message>(&buffer).expect("Couldn't deserialize response!");
    match resp {
        Message::Ack(v) => match v {
            true => {
                println!("Job accepted!");
                let mut buffer = Vec::new();
                reader.read_until(0xA, &mut buffer)?;
                buffer.pop();
                let resp = deserialize::<Message>(&buffer).expect("Couldn't deserialize response!");
                match resp {
                    Message::Success(tup) => {
                        println!("Job finished! {:?}", tup);
                        exit(0);
                    }
                    Message::Failure(tup) => {
                        eprintln!("Job failed on server! {:?}", tup);
                        exit(1);
                    }
                    _ => {
                        eprintln!("Unhandled job response");
                        exit(1);
                    }
                }
            }
            false => {
                eprintln!("Job not accepted!");
                exit(1);
            }
        },
        _ => {
            eprintln!("Job not accepted!");
            exit(1);
        }
    }
}
